"""
This file has been generated by ipv4-allocations-update.py program.

If you need it updated, override this file content with ipv4-allocations-update.py output.
"""

from ipaddress import IPv4Network


AllocationsT = dict[IPv4Network, dict[str, str]]


IPV4_ALLOCATIONS: AllocationsT = {
    IPv4Network("1.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("14.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("27.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("36.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("39.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("42.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("49.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("58.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("60.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("101.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("103.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("106.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("110.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("112.0.0.0/5"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("120.0.0.0/6"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("124.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("126.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("133.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("150.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("153.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("163.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("171.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("175.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("180.0.0.0/8"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("182.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("202.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("210.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("218.0.0.0/7"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("220.0.0.0/6"): {
        "rdap": "https://rdap.apnic.net/",
        "whois": "whois.apnic.net",
    },
    IPv4Network("2.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("5.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("25.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("31.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("37.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("46.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("51.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("53.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("57.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("62.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("77.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("78.0.0.0/7"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("80.0.0.0/4"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("109.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("141.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("145.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("151.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("176.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("178.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("185.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("188.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("193.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("194.0.0.0/7"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("212.0.0.0/7"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("217.0.0.0/8"): {
        "rdap": "https://rdap.db.ripe.net/",
        "whois": "whois.ripe.net",
    },
    IPv4Network("3.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("4.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("6.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("8.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("11.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("12.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("15.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("16.0.0.0/5"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("24.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("26.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("28.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("30.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("32.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("38.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("40.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("44.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("47.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("48.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("50.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("52.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("54.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("56.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("63.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("64.0.0.0/5"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("72.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("76.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("96.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("100.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("104.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("107.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("108.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("128.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("132.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("134.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("136.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("140.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("142.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("144.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("146.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("148.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("152.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("155.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("156.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("160.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("162.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("164.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("168.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("170.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("172.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("174.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("184.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("192.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("198.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("204.0.0.0/6"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("208.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("214.0.0.0/7"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("216.0.0.0/8"): {
        "rdap": "https://rdap.arin.net/registry",
        "whois": "whois.arin.net",
    },
    IPv4Network("41.0.0.0/8"): {
        "rdap": "https://rdap.afrinic.net/rdap/",
        "whois": "whois.afrinic.net",
    },
    IPv4Network("102.0.0.0/8"): {
        "rdap": "https://rdap.afrinic.net/rdap/",
        "whois": "whois.afrinic.net",
    },
    IPv4Network("105.0.0.0/8"): {
        "rdap": "https://rdap.afrinic.net/rdap/",
        "whois": "whois.afrinic.net",
    },
    IPv4Network("154.0.0.0/8"): {
        "rdap": "https://rdap.afrinic.net/rdap/",
        "whois": "whois.afrinic.net",
    },
    IPv4Network("196.0.0.0/7"): {
        "rdap": "https://rdap.afrinic.net/rdap/",
        "whois": "whois.afrinic.net",
    },
    IPv4Network("177.0.0.0/8"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("179.0.0.0/8"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("181.0.0.0/8"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("186.0.0.0/7"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("189.0.0.0/8"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("190.0.0.0/7"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
    IPv4Network("200.0.0.0/7"): {
        "rdap": "https://rdap.lacnic.net/rdap/",
        "whois": "whois.lacnic.net",
    },
}


__all__ = ("IPV4_ALLOCATIONS", "AllocationsT")
